@@ -35,7 +35,7 @@ int am_traced(char *name);
 
 
 
-#endif __CE_TRACE_H__
+#endif /* __CE_TRACE_H__ */
 /********************************************************************/
 
 
