@@ -1,12 +1,3 @@
-//**************************************************************
-//*
-//*  Ensemble, (Version 0.70p1)
-//*  Copyright 2000 Cornell University
-//*  All rights reserved.
-//*
-//*  See ensemble/doc/license.txt for further information.
-//*
-//**************************************************************
 package ensemble.hot;
 
 import java.io.*;
@@ -24,7 +15,7 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
     private Hot_IO_Controller hioc;
     private Process ensOutboardProcess;
     private Socket ensOutboardSocket;
-    private static boolean debug;
+    private static boolean debug = true;
     private static boolean use_byteStream;   // flag to use byte stream communic.
 	
     /**
@@ -77,7 +68,7 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
 	use_byteStream = use_BS;
 	ServerSocket s = null;
 	while (port == 0) {
-            port = (int)((Math.random() * 3000) + 5000);
+	    port = (int)((Math.random() * 3000) + 5000);
 	    try {
 		s = new ServerSocket(port);
 	    } catch (Exception e) {
@@ -101,6 +92,7 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
     */
     public Hot_Ensemble(int port) {
 	try {
+	    System.out.println("outboard: starting up client side" + port);
 	    ensOutboardSocket = new Socket(InetAddress.getLocalHost(), port);
 	    hioc = new Hot_IO_Controller(ensOutboardSocket.getInputStream(),
 					 ensOutboardSocket.getOutputStream());
@@ -120,13 +112,13 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
     Sets up child process, creates the new IO Controller from this child process
     */
     private void init(int port) {
-//	String outboard = "outboard -tcp_channel -tcp_port " + Integer.toString(port);
+//	String outboard = "outboard -tcp_port " + Integer.toString(port);
 
-	//String outboard = "outboard -modes DEERING:UDP -deering_port 12222  -tcp_channel -tcp_port " +  Integer.toString(port);
+	//String outboard = "outboard -modes DEERING:UDP -deering_port 12222 -tcp_port " +  Integer.toString(port);
 	
-	String outboard = "outboard -verbose -tcp_channel -tcp_port " + 
+	String outboard = "outboard -verbose -tcp_port " + 
 	  Integer.toString(port) + " -trace HOT_OUTBOARD";
-	//String outboard="outboard -deering_port 77777 -tcp_channel -tcp_port " +
+	//String outboard="outboard -deering_port 77777 -tcp_port " +
 	//  Integer.toString(port);
 
 	debug = false;
@@ -138,7 +130,7 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
 		// Added by TC - wait for forked process to start up.
 		try {
 		    System.out.println("Waiting for the outboard process to start");
-		    Thread.sleep(2500);
+		    Thread.sleep(5000);
 		}
 		catch (InterruptedException e) {
 		    System.out.println(e);
@@ -210,6 +202,9 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
 	    hioc.write_string(jops.params);
 	    hioc.write_bool(jops.client);
 	    hioc.write_bool(jops.debug);
+	    hioc.write_string(jops.princ);
+	    hioc.write_string(jops.key);
+	    hioc.write_bool(jops.secure);
 	    try {
 	        hioc.end_write();
 	    } 
@@ -343,9 +338,6 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
     }
 
 
-    /**
-    NOT SUPPORTED CURRENTLY IN THE ML
-    */
     public Hot_Error Suspect(Hot_GroupContext gc, Hot_Endpoint[] suspects) {
 	if (!outboardValid)
 	    return new Hot_Error(55,"Outboard process is not valid!");
@@ -371,6 +363,33 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
     }
 
 
+    /**
+    Infrom Ensemble that state-transfer is complete. 
+    */
+    public Hot_Error XferDone(Hot_GroupContext gc) {
+	if (!outboardValid)
+	    return new Hot_Error(55,"Outboard process is not valid!");
+	Hot_Error err = null;
+	synchronized(WriteMutex) {
+            synchronized(CriticalMutex) {
+		if (gc.leaving) {
+		    Panic("hot_ens_xfer_done: this member is already leaving");
+		}
+		gc.leaving = true;
+            }
+	    // write the downcall.
+	    hioc.begin_write();
+	    hioc.write_hdr(gc, hioc.DN_XFERDONE);
+	    try {
+	        hioc.end_write();
+	    } 
+	    catch (IOException e) {
+		err = new Hot_Error(0, e.toString());
+	    }	        
+	}
+	return err;
+    }
+
     /**
     Change the protocol used by the group specified by the Hot_GroupContext 
     to the protocol specified by the String
@@ -462,6 +481,33 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
     }
 
 
+    /**
+    Request Ensmeble to rekey the group. 
+    */
+    public Hot_Error Rekey(Hot_GroupContext gc) {
+	if (!outboardValid)
+	    return new Hot_Error(55,"Outboard process is not valid!");
+	Hot_Error err = null;
+	synchronized(WriteMutex) {
+	    synchronized(CriticalMutex) {
+		if (gc.leaving) {
+		    Panic("hot_ens_Rekey: member is leaving");
+		}
+	    }
+            // write the downcall
+	    hioc.begin_write();
+            hioc.write_hdr(gc, hioc.DN_REKEY);
+	    try {
+	        hioc.end_write();
+            } 
+            catch (IOException e) {
+                err = new Hot_Error(0, e.toString());
+            }	        
+	}
+	return err;
+    }
+
+
     /** 
     Receive new view callback and print out info
     */
@@ -713,8 +759,8 @@ public class Hot_Ensemble extends java.lang.Object implements java.lang.Runnable
 		case 3:    // cb_Heartbeat
 		    cb_Heartbeat(groupID[0]);
 		    break;
-		case 4:    // cb_Block
-		    cb_Block(groupID[0]);
+		case 4:    // cb_Block	
+	    cb_Block(groupID[0]);
 		    break;
 		case 5:    // cb_Exit
 		    cb_Exit(groupID[0]);
