@@ -1,13 +1,4 @@
 (**************************************************************)
-(*
- *  Ensemble, 1.10
- *  Copyright 2001 Cornell University, Hebrew University
- *  All rights reserved.
- *
- *  See ensemble/doc/license.txt for further information.
- *)
-(**************************************************************)
-(**************************************************************)
 (* CHK_REKEY.ML *)
 (* Author: Ohad Rodeh, 3/99 *)
 (* Sanity test for the rekeying *)
@@ -119,7 +110,7 @@ let l args vf = Layer.hdr init hdlrs None NoOpt args vf
 
 let _ = 
   Param.default "chk_rekey_degree" (Param.Int 4);
-  Elink.layer_install name l
+  Layer.install name l
 
 (**************************************************************)
 
