@@ -3,4 +3,4 @@
 (* Author: Mark Hayden, 3/97 *)
 (**************************************************************)
 
-val f : Mbuf.t -> (int -> Iovecl.t -> unit) Route.t
+val f : unit -> (Trans.rank -> Trans.seqno -> Iovecl.t -> unit) Route.t
