@@ -1,13 +1,4 @@
 (**************************************************************)
-(*
- *  Ensemble, 1.10
- *  Copyright 2001 Cornell University, Hebrew University
- *  All rights reserved.
- *
- *  See ensemble/doc/license.txt for further information.
- *)
-(**************************************************************)
-(**************************************************************)
 (* ARRAYF.ML : functional arrays *)
 (* Author: Mark Hayden, 4/95 *)
 (**************************************************************)
@@ -146,10 +137,11 @@ let choose a k =
   in loop k []
 
 (**************************************************************)
-(* Hack *)
-let sendv = Arraye.sendv
-let sendtov = Arraye.sendtov
-let sendtosv = Arraye.sendtosv
-let sendtovs = Arraye.sendtovs
-let eth_sendtovs = Arraye.eth_sendtovs
+(* This should be used with care, it is unsafe. 
+*)
+let to_array_break x = (Obj.magic x : 'a array)
+let of_array_break x = (Obj.magic x : 'a t)
+
+let of_array_map = Arraye.of_array_map
+let to_array_map = Arraye.to_array_map
 (**************************************************************)
